"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = extensionResolver;

var _path = _interopRequireDefault(require("path"));

var _resolve = _interopRequireDefault(require("resolve"));

var _package = require("../package");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DEFAULT_EXTENSIONS = ['.node.mjs', '.mjs', '.node.js', '.js', '.node.ts', '.ts', '.node.tsx', '.tsx', '.json', '.node.jsx', '.jsx', '.node'];

function extensionResolver({
  types
}, {
  extensions = DEFAULT_EXTENSIONS,
  resolveOptions = {}
}) {
  return {
    name: _package.name,
    visitor: {
      Program: {
        enter(programPath, state) {
          const {
            file: {
              opts: {
                filename
              }
            }
          } = state;

          function replaceSource(source) {
            if (!types.isStringLiteral(source)) {
              return;
            }

            const sourcePath = source.node.value;

            if (sourcePath[0] !== '.') {
              return;
            }

            const basedir = _path.default.dirname(filename);

            const resolvedPath = _path.default.relative(basedir, _resolve.default.sync(sourcePath, { ...resolveOptions,
              basedir,
              extensions
            }));

            source.replaceWith(types.stringLiteral(resolvedPath[0] === '.' ? resolvedPath : `./${resolvedPath}`));
          }

          programPath.traverse({
            CallExpression(declaration) {
              const callee = declaration.get('callee');

              if (!types.isIdentifier(callee) || callee.node.name !== 'require') {
                return;
              }

              const args = declaration.get('arguments');

              if (args.length !== 1) {
                return;
              }

              replaceSource(args[0]);
            },

            ImportDeclaration(declaration) {
              replaceSource(declaration.get('source'));
            }

          }, state);
        }

      }
    }
  };
}
//# sourceMappingURL=index.js.map